'use strict';
var forEach = require('lodash.foreach');
var constants = require('./constants');
var defaultStorage = require('./defaults/asyncLocalStorage');

module.exports = function persistStore(store, config, cb) {
  // defaults
  config = config || {};
  var blacklist = config.blacklist || [];
  var whitelist = config.whitelist || false;
  var rehydrateAction = config.rehydrateAction || defaultRehydrateAction;
  var completeAction = config.completeAction || defaultCompleteAction;
  var serialize = config.serialize || defaultSerialize;
  var deserialize = config.deserialize || defaultDeserialize;
  var transforms = config.transforms || [];
  var storage = config.storage || defaultStorage;
  var debounce = config.debounce || 33;

  // initialize values
  var timeIterator = null;
  var lastState = store.getState();
  var purgeMode = false;
  var restoreCount = 0;
  var completionCount = 0;
  var storesToProcess = [];

  forEach(lastState, function (s, key) {
    if (whitelistBlacklistCheck(key)) {
      return;
    }
    restoreCount += 1;
    setImmediate(function () {
      restoreKey(key, function () {
        completionCount += 1;
        if (completionCount === restoreCount) {
          rehydrationComplete();
        }
      });
    });
  });
  if (restoreCount === 0) {
    rehydrationComplete();
  }

  // store state to disk
  store.subscribe(function () {
    // clear unfinished timeIterator if exists
    if (timeIterator !== null) {
      clearInterval(timeIterator);
    }

    var state = store.getState();
    forEach(state, function (subState, key) {
      if (whitelistBlacklistCheck(key)) {
        return;
      }
      if (lastState[key] === state[key]) {
        return;
      }
      if (storesToProcess.indexOf(key) !== -1) {
        return;
      }
      storesToProcess.push(key);
    });

    // time iterator (read: debounce)
    timeIterator = setInterval(function () {
      if (storesToProcess.length === 0) {
        clearInterval(timeIterator);
        return;
      }

      var key = createStorageKey(storesToProcess[0]);
      var endState = transforms.reduce(function (subState, transformer) {
        return transformer['in'](subState);
      }, state[storesToProcess[0]]);
      if (typeof endState !== 'undefined') {
        var serial = serialize(endState);
        storage.setItem(key, serial, warnIfSetError(key));
      }
      storesToProcess.shift();
    }, debounce);

    lastState = state;
  });

  function whitelistBlacklistCheck(key) {
    if (whitelist && whitelist.indexOf(key) === -1) {
      return true;
    }
    if (blacklist.indexOf(key) !== -1) {
      return true;
    }
    return false;
  }

  function restoreKey(key, cb) {
    storage.getItem(createStorageKey(key), function (err, serialized) {
      if (err && process.env.NODE_ENV !== 'production') console.warn('Error restoring data for key:', key, err);else rehydrate(key, serialized, cb);
    });
  }

  function rehydrate(key, serialized, cb) {
    var state = null;
    try {
      var data = deserialize(serialized);
      state = transforms.reduceRight(function (subState, transformer) {
        return transformer.out(subState);
      }, data);
    } catch (err) {
      if (process.env.NODE_ENV !== 'production') console.warn('Error restoring data for key:', key, err);
      storage.removeItem(key, warnIfRemoveError(key));
    }

    if (state !== null) {
      if (purgeMode === '*' || Array.isArray(purgeMode) && purgeMode.indexOf(key) !== -1) {
        return;
      }
      store.dispatch(rehydrateAction(key, state));
    }
    cb();
  }

  function rehydrationComplete() {
    store.dispatch(completeAction());
    cb && cb();
  }

  return {
    rehydrate: rehydrate,
    purge: function purge(keys) {
      purgeMode = keys;
      forEach(keys, function (key) {
        storage.removeItem(createStorageKey(key), warnIfRemoveError(key));
      });
    },
    purgeAll: function purgeAll() {
      purgeMode = '*';
      storage.getAllKeys(function (err, keys) {
        if (err) {
          console.warn(err);
        }
        forEach(keys, function (key) {
          if (key.indexOf(constants.keyPrefix) === 0) {
            storage.removeItem(key, warnIfRemoveError(key));
          }
        });
      });
    }
  };
};

function warnIfRemoveError(key) {
  return function removeError(err) {
    if (err && process.env.NODE_ENV !== 'production') {
      console.warn('Error storing data for key:', key, err);
    }
  };
}

function warnIfSetError(key) {
  return function setError(err) {
    if (err && process.env.NODE_ENV !== 'production') {
      console.warn('Error storing data for key:', key, err);
    }
  };
}

function createStorageKey(key) {
  return constants.keyPrefix + key;
}

function defaultRehydrateAction(key, data) {
  return {
    type: constants.REHYDRATE,
    key: key,
    payload: data
  };
}

function defaultCompleteAction() {
  return {
    type: constants.REHYDRATE_COMPLETE
  };
}

function defaultSerialize(data) {
  return JSON.stringify(data);
}

function defaultDeserialize(serial) {
  return JSON.parse(serial);
}